####################################################################################################
#  Bayesian Estimation of Sparsely Heterogenous Panel Models
#  Empirical Analysis, Balanced Panel
#####################################################################################################

#* This script calculates MSE for each estimator

####################################################################################################
#                             Part 0: INTRO
####################################################################################################

cd()
wd = "$(pwd())/Dropbox/SparseHetero/Software/ReplicationFile_v231019/Empirics/Empirics_Balanced/"
cd(wd)

version = "_balanced"

# directory of LPS
wd_results = string(wd, "Results/")
# create lps folder under Results
wd_mse = string(wd_results, "MSE/")
if !isdir(wd_mse)
     mkdir(wd_mse)
end

# directory of data
wd_data = string(wd, "data", version, "/")

# load packages (install them with Pkg.add() if needed)
using DelimitedFiles, MAT, Distributions

####################################################################################################
#                             Part 1: INITALIZATION
####################################################################################################

## Data settings
# starting year
yr0 = 67
# end year (= 92(96) if last year = 1993(1997))
yr1 = 96
# number of observations in the sample
# bigT = [4, 6, 10, 15, 20, 25, 28]
bigT = 20

####################################################################################################
#                             Part 3: DRAW HISTOGRAMS
####################################################################################################

estimator_list = ["M2","M2_homosk","M2_HIP", "M2_RIP"]

for j in eachindex(estimator_list)

     estimator = estimator_list[j]

     # folder of jld2 outout
     wd_mat = string(wd, "Estimates/", estimator, "/")

     output = zeros(200,3)
     counter = 0

     for T = bigT
          # T = 20
          T2 = T + 2

          for t_smpl = (yr0 + T2 - 1):yr1
               # t_smpl = (yr0 + T2 - 1)
               counter += 1

               label = "T$(T)_tau$(t_smpl)_"

               # load forecast in MAT file
               temp = matread(wd_mat * "$(label)results.mat")
               fcst = temp["post_mean"]["fcst"]

               # load the data
               raw = readdlm(string(wd_data, label, "data.txt"), ' ')

               id = unique(raw[:,1])
               N  = length(id)

               Y = zeros(T2, N)
               h = zeros(T2, N)
               for t = 1:T2
                    for i = 1:N
                         selected = (abs.(raw[:,1] .- id[i]) .< 0.05) .& (abs.(raw[:,2] .- (t_smpl-T2+t)) .< 0.05)
                         Y[t,i] = raw[selected, 3][1]
                         h[t,i] = raw[selected, 5][1]
                    end
               end

               y_target = Y[end,:]

               output[counter,:] = [T t_smpl mean((fcst - y_target).^2)]

          end

     end

     output = output[1:counter, :]
     writedlm( wd_mse * "MSE_$estimator.csv",  output, ',')
end

